@@ -5,8 +5,8 @@
 #' \code{cn} generates the control net for the given B-spline function.  There
 #' are several methods for building a control net.
 #'
-#' @param x a \code{cpr_bs} object
-#' @param ... arguments passed to the regression method
+#' @param x a \code{cpr_bt} object
+#' @param ... pass through
 #'
 #' @return a \code{cpr_cn} object.  This is a list with the following elements.
 #' Some of the elements are omitted when the using the \code{cn.cpr_bt} method.
@@ -22,11 +22,23 @@
 #'  mixed effects model was used.}
 #'  \item{vcov}{The variance-covariance matrix for the \code{coefficients}}
 #'  \item{loglik}{The log-likelihood for the regression model}
-#'  \item{rmse}{The root mean squared error for the regression models}
+#'  \item{rse}{the residual standard error for the regression models}
 #'  }
 #'
+#' @seealso \code{\link{summary.cpr_cn}}, \code{\link{cnr}},
+#' \code{\link{plot.cpr_cn}} for plotting control nets
+#'
+#' @examples
+#'
+#' acn <- cn(log10(pdg) ~
+#'               btensor(   x     = list(day, age)
+#'                       , df     = list(30, 4)
+#'                       , bknots = list(c(-1, 1), c(44, 53))
+#'               )
+#'            , data = spdg)
+#' str(acn, max.level = 1)
+#'
 #' @export
-#' @rdname cn
 cn <- function(x, ...) {
   UseMethod("cn")
 }
@@ -34,7 +46,7 @@ cn <- function(x, ...) {
 #' @export
 #' @rdname cn
 #' @param theta a vector of (regression) coefficients, the ordinates of the
-#'        control net.
+#' control net.
 cn.cpr_bt <- function(x, theta, ...) {
   xi_stars <- lapply(attr(x, "bspline_list"), attr, which = "xi_star")
 
@@ -42,32 +54,39 @@ cn.cpr_bt <- function(x, theta, ...) {
     list(cn      = data.frame(cbind(do.call(expand.grid, xi_stars), theta)),
          bspline_list = attr(x, "bspline_list"),
          call    = match.call(),
-         keep_fit = NA,
-         fit     = NA,
-         loglik  = NA,
-         rmse    = NA)
+         keep_fit = NULL,
+         fit    = NULL,
+         theta = NULL,
+         coefficients = NULL,
+         vcov = NULL,
+         vcov_theta = NULL,
+         loglik = NULL,
+         rss    = NULL,
+         rse    = NULL)
   class(out) <- c("cpr_cn", class(out))
   out
 }
 
 #' @export
 #' @rdname cn
-#' @param formula a formula that is appropriate for regression method being
-#'        used.
+#' @param formula a formula that is appropriate for regression method being used.
 #' @param data a required \code{data.frame}
 #' @param method the regression method such as \code{\link[stats]{lm}},
-#'        \code{\link[stats]{glm}}, \code{\link[lme4]{lmer}}, etc.
+#' \code{\link[stats]{glm}}, \code{\link[lme4]{lmer}}, etc.
+#' @param method.args a list of additional arguments to pass to the regression
+#' method.
 #' @param keep_fit (logical, defaults to \code{FALSE}).  If \code{TRUE} the
 #' regression model fit is retained and returned in the the \code{fit} element.
-#' If \code{FALSE} the regression model is not saved and the \code{fit} element will be \code{NA}.
+#' If \code{FALSE} the regression model is not saved and the \code{fit} element
+#' will be \code{NA}.
 #' @param check_rank (logical, defaults to \code{TRUE}) if TRUE check that the
 #' design matrix is full rank.
-cn.formula <- function(formula, data, method = stats::lm, ..., keep_fit = FALSE, check_rank = TRUE) {
+cn.formula <- function(formula, data, method = stats::lm, method.args = list(), keep_fit = TRUE, check_rank = TRUE, ...) {
+
   # check for some formula specification issues
-  fterms <- stats::terms(formula)
-  fterms
-  if (sum(grepl("btensor", attr(fterms, "term.labels"))) != 1) {
-    stop("cpr::btensor() must appear once, with no effect modifiers, on the right hand side of the formula.")
+  rhs_check <- grepl("btensor", attr(stats::terms(formula), "term.labels"))
+  if ( !rhs_check[1] | any(rhs_check[-1]) ) {
+    stop("btensor() must appear first, once, and with no effect modifiers, on the right hand side of the formula.")
   }
 
   # this function will add f_for_use and data_for_use into this environment
@@ -75,20 +94,18 @@ cn.formula <- function(formula, data, method = stats::lm, ..., keep_fit = FALSE,
   generate_cp_formula_data(formula, data)
 
   regression <- match.fun(method)
-  cl <- as.list(match.call())
-  cl <- cl[-c(1, which(names(cl) %in% c("method", "keep_fit", "check_rank")))]
-  cl$formula <- as.name("f_for_use")
-  cl$data <- as.name("data_for_use")
+  cl <- list(formula = as.name("f_for_use"), data = as.name("data_for_use"))
+  cl <- c(cl, method.args)
 
   fit <- do.call(regression, cl)
+  Bmat <- stats::model.frame(fit)[[2]]
+  COEF_VCOV <- coef_vcov(fit, theta_idx = seq(1, ncol(Bmat), by = 1))
 
   if (check_rank) {
     m <- stats::model.matrix(lme4::nobars(f_for_use), data_for_use)
-    if (matrix_rank(m) != ncol(m) | any(is.na(BETA(fit)))) {
-      warning("Design Matrix is rank deficient. keep_fit being set to TRUE.",
-              call. = FALSE,
-              immediate. = TRUE)
-    keep_fit <- TRUE
+    if (matrix_rank(m) != ncol(m) | any(is.na(COEF_VCOV$coef))) {
+      keep_fit <- TRUE
+      warning("Design Matrix is rank deficient. keep_fit being set to TRUE.")
     }
   }
 
@@ -96,50 +113,31 @@ cn.formula <- function(formula, data, method = stats::lm, ..., keep_fit = FALSE,
   cl[[1]] <- as.name("cn")
   cl <- as.call(cl)
 
-  Bmat <- eval(extract_cpr_bsplines(formula), data, environment(formula))
-  xi_stars <- lapply(attr(Bmat, "bspline_list"), attr, which = "xi_star")
+  out <- cn.cpr_bt(Bmat, COEF_VCOV$theta)
 
-  out <-
-    list(cn      = data.frame(cbind(do.call(expand.grid, xi_stars),
-                                 theta   = as.vector(theta(fit)))),
-         bspline_list = attr(Bmat, "bspline_list"),
-         call    = cl,
-         keep_fit = keep_fit,
-         fit     = if (keep_fit) { fit } else { NA },
-         coefficients = BETA(fit),
-         vcov = SIGMA(fit),
-         loglik  = loglikelihood(fit),
-         rmse    = sqrt(mean(stats::residuals(fit)^2)))
+  # update elements of the cpr_bs object
+  if (keep_fit) {
+    out[["fit"]] <-  fit
+  }
+
+  out[["call"]]         <- cl
+  out[["keep_fit"]]     <- keep_fit
+  out[["theta"]]        <- COEF_VCOV$theta
+  out[["vcov_theta"]]   <- COEF_VCOV$vcov_theta
+  out[["coefficients"]] <- COEF_VCOV$coef
+  out[["vcov"]]         <- COEF_VCOV$vcov
+  out[["loglik"]]       <- loglikelihood(fit)
+  out[["rss"]]          <- sum(stats::residuals(fit)^2)
+  out[["rse"]]          <- sqrt(sum(stats::residuals(fit)^2) / (nrow(data) - length(COEF_VCOV$coef)))
+
+  out
   class(out) <- c("cpr_cn", class(out))
 
   out
 }
 
-#' @method print cpr_cn
 #' @export
-#' @rdname cn
 print.cpr_cn <- function(x, ...) {
   print(x$cn, ...)
-}
-
-#' @export
-#' @param object a \code{cpr_cn} object
-#' @rdname cn
-summary.cpr_cn <- function(object, ...) {
-  iknots <- lapply(object$bspline_list, attr, which = "iknots")
-  names(iknots) <- paste0("iknots", seq_along(iknots))
-
-  out <-
-    data.frame(dfs        = length(object$cn$theta),
-               loglik     = object$loglik,
-               rmse       = object$rmse)
-
-  for(i in seq_along(iknots)) {
-    nm <- names(iknots)[i]
-    out[[paste0("n_", nm)]] <- length(iknots[[i]])
-    out[[nm]] <- I(iknots[i])
-  }
-
-  out
-
+  invisible(x)
 }
